import { Construct } from "constructs";
import { SSTConstruct } from "./Construct.js";
export interface ParameterProps {
    /**
     * Value of the parameter
     */
    value: string;
}
/**
 * The `Parameter` construct is a higher level CDK construct that makes it easy to manage app environment variables.
 *
 * @example
 * ### Using the minimal config
 *
 * ```js
 * import { Config } from "@serverless-stack/resources";
 *
 * new Config.Parameter(stack, "TABLE_NAME", table.tableName);
 * ```
 */
export declare class Parameter extends Construct implements SSTConstruct {
    readonly id: string;
    readonly name: string;
    readonly value: string;
    constructor(scope: Construct, id: string, props: ParameterProps);
    /** @internal */
    getConstructMetadata(): {
        type: "Parameter";
        data: {
            name: string;
        };
    };
    /** @internal */
    getFunctionBinding(): {
        clientPackage: string;
        variables: {
            value: {
                environment: string;
                parameter: string;
            };
        };
        permissions: {};
    };
    static create<T extends Record<string, any>>(scope: Construct, parameters: T): { [key in keyof T]: Parameter; };
}
